import requests

from . import itime


def __init__(params: dict):
    return IsddcAPI(**params)


class IsddcAPI:
    token_expires_time_point = 0

    uri = dict(
        metrics='/v2.0/metrics',
        inventories='/v2.0/inventories'
    )

    def __init__(
            self,
            api: str,
            keystone: str,
            username: str,
            password: str,
            tenantid: str,
            token_expires_time: int,
            request_timeout: int = 30
    ):
        self.api = api
        self.keystone = keystone
        self.username = username
        self.password = password
        self.tenantid = tenantid
        self.token_expires_time = token_expires_time
        self.request_timeout = request_timeout

        self.auth_api = f'http://{self.keystone}/v3/auth/tokens'

        for name, uri in self.uri.items():
            self.uri[name] = f'http://{self.api}{uri}'

        self.headers = {
            'Content-Type': 'application/json',
            'X-Auth-Token': None
        }

    def __getattribute__(self, name):
        if name == 'send' and self.token_expires_time_point < itime.Now.float:
            self.headers['X-Auth-Token']: str = requests.post(
                url=self.auth_api,
                json={'auth': {
                    'identity': {
                        'methods': ['password'],
                        'password': {'user': {
                            'domain': {'id': 'default'},
                            'name': self.username,
                            'password': self.password}}},
                    'scope': {'project': {
                        'id': self.tenantid,
                        'domain': {'id': 'default'}}}}}
            ).headers['X-Subject-Token']
            self.token_expires_time_point = itime.Future(self.token_expires_time).float

        return super().__getattribute__(name)

    def send(
            self,
            data: dict,
            dtype: 'enum(metrics, inventories)'
    ) -> requests.Response:
        return requests.post(
            url=self.uri[dtype],
            json=data,
            headers=self.headers,
            timeout=self.request_timeout
        )
