from influxdb import InfluxDBClient

from . import idec
from . import idict
from . import ioverfunc

__default__: InfluxDBClient


@idec.TryExcept(Exception, mark='InitInflux')
@idec.Retry('InitInflux', count=10, cycle=10)
def __init__(params: dict) -> InfluxDBClient:
    client = InfluxDBClient(**params)
    ioverfunc.add(client.close)
    return client


def query(
        sql: str,
        one: bool = False,
        epoch: str = None,
        datastyle: 'enum(dict, list)' = dict,
        client: InfluxDBClient = None,
        *a, **kw) -> list or dict:
    """
    Encapsulate the 'InfluxDBClient.query' method.

    Help:
        Turn off the highlighting of the PyCharm SQL
        statement: `File -> Editor -> Inspections -> SQL`

    :param one: Returns a piece of data.
    :param epoch: The time format.
    :param datastyle: The data type returned, dict or str
    :param client: Specified database, the default is `__default__`.
    """
    queryset = (client or __default__).query(sql, epoch=epoch, *a, **kw)

    if not queryset:
        return {} if datastyle is dict and one else []

    if datastyle is dict:
        data = idict([[dict(
            (qst['columns'][i], val[i])
            for i in range(len(val)))
            for val in qst['values']]
            for qst in queryset.raw['series']])
    else:
        data = [series['values'] for series in queryset.raw['series']]

    if 'GROUP BY' not in sql:
        data = data[0]

    return data[0] if one else data


def write_points(
        data: list,
        timep: str = None,
        client: InfluxDBClient = None,
        *a, **kw
) -> bool:
    return (client or __default__).write_points(
        points=data,
        time_precision=timep,
        *a, **kw)
